
/* 
 * RAPID_P5
 * Example demonstrating how to create a RAPID program from Processing
 * Requires toxiclibs: http://toxiclibs.org/
 * 
 * Created by Karl D.D. Willis
 * for the FUTURE CNC course at CMU
 * http://futurecnc.code.arc.cmu.edu/
 *
 *
 */

import toxi.geom.*;
import toxi.geom.mesh.*;
import toxi.processing.*;
import processing.opengl.*;

// Room and tool import
TriangleMesh room, tool;

// Tool locations (millimeters) and rotations (degrees)
PVector[] locs = new PVector[10];
PVector[] rots = new PVector[10];

// Display stuff
ToxiclibsSupport gfx;
PFont myFont;


void setup() {
  
  size(800, 600, OPENGL);

  // Load the room and tool to display in our 'visualization'
  room = (TriangleMesh) new STLReader().loadBinary(sketchPath("data/Room.stl"), STLReader.TRIANGLEMESH);
  tool = (TriangleMesh) new STLReader().loadBinary(sketchPath("data/MyTool.stl"), STLReader.TRIANGLEMESH);

  // Make some points for the robot to go to
  // and some rotations (vectors) using Euler Angles
 //  http://en.wikipedia.org/wiki/Euler_Angles
 
 //  Start location is directly in front of the base
 //  specifically, 600mm in front of the robot and 300mm up off the table
 PVector startLoc = new PVector(600, 0, 500);
 
  for (int i=0; i<locs.length; i++) {
    // Create a generic step size
    int step = i * 10;

    // Simple example - horizontal movement, constant rotation
    locs[i] = new PVector(startLoc.x, step * 3, startLoc.z);
    rots[i] = new PVector(0, 90, 0);
    
    // More complex example
    //locs[i] = new PVector(startLoc.x - step, step * 5, startLoc.z + step);
    //rots[i] = new PVector(step, 90 + step, step * 2);
  }
  

  // Display stuff
  gfx = new ToxiclibsSupport(this);
  hint(ENABLE_NATIVE_FONTS);
  myFont = createFont( PFont.list()[3], 12, false);
  textFont(myFont);
}

void draw() {

  background(51);
  fill(255, 255, 255);
  lights();
  
  pushMatrix();
  // Move to the middle of the screen
  translate(width/2, height/2, 0);

  // Mouse orbiting
  rotateX(-radians(mouseY));
  rotateY(-radians(mouseX));
  // Align with the front of the table
  rotateX(-HALF_PI);
  rotateZ(-HALF_PI);

  // Scale down to fit the model inside the window
  scale(0.2);
  // Flip the y to match with robot studio coordinates
  scale(1, -1, 1);

  // ROOM
  // Axes
  gfx.origin(new Vec3D(), 300);
  noStroke();
  // Room model
  gfx.mesh(room);


  // TOOLS
  for (int i=0; i<10; i++) {
    pushMatrix();
    translate(locs[i].x, locs[i].y, locs[i].z);
    rotateZ(radians(rots[i].z));
    rotateY(radians(rots[i].y));
    rotateX(radians(rots[i].x));    
    // Draw the axes
    // The Z (blue) axis indicates the tool direction    
    gfx.origin(new Vec3D(), 300);
    fill(255, 0, 0);
    noStroke();
    // Slow to draw this...
    gfx.mesh(tool);
    popMatrix();
  }
  
  // TEXT DISPLAY
  hint(DISABLE_DEPTH_TEST);
  popMatrix();
  fill(255);
  text("Click to export a RAPID program", 10, 15);
  hint(ENABLE_DEPTH_TEST);
}


void mouseReleased() {  
  writeRapid();
}

/* Create a RAPID program in its own folder in the sketch directory */
void writeRapid() {
  // Specify that we move to the home location first before anything else
  RapidWriter rw = new RapidWriter(true);
  // Location and orientation of the tool center point
  // The orientation quaternion can be retreived from RobotStudio
  rw.setToolData(new PVector(0, 0, 127), 1, 0, 0, 0);
  // Add the MoveJ locations and orientations
  for (int i=0; i<locs.length; i++) {
    rw.addMoveJ(locs[i], rots[i]);
  }
  // Write the program out with this name  
  rw.write("MyRapidProgram");
}

